<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:plm="http://www.plmxml.org/Schemas/PLMXMLSchema"
	xmlns:cim="http://www.coolemall.eu/DEBBComponent" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	xmlns:fun="http://whatever">

	<!-- Importing xslt for translating debb components -->
	<xsl:import href="DEBBComponentTranslator.xsl" />

	<!-- Information for mapping of DEBB classes into DCWoRMS computingResource 
		classes -->
	<xsl:param name="computingResourceClasses" select="computingResourceClasses" />

	<!-- Information for mapping of resource classes into DCWoRMS resourceUnits 
		classes -->
	<!-- (Needed to pass to DEBB Components xslt) -->
	<xsl:param name="resourceUnitClasses" select="resourceUnitClasses" />

	<!-- Information from which level the translator should start moving information to templates -->
	<xsl:variable name="templateStartLevel" select="'Node'" />

	<!-- Specification of the output document -->
	<xsl:output method="xml" version="1.0" encoding="UTF-8"
		indent="yes" />

	<!-- Root of the PLM XML document -->
	<xsl:variable name="rootDoc" select="/" />

	<!-- Default DEBBComponents file name (used if it is not specified in PLM 
		XML file) -->
	<xsl:variable name="defaultDEBBComponentsFileName" select="'./example/DEBBComponents.xml'" />


	<xsl:template match="/">
		<xsl:element name="environment">
			<xsl:attribute name="xsi:noNamespaceSchemaLocation">../DCWormsResSchema.xsd</xsl:attribute>
			
			<xsl:element name="resources">
				<!-- Iterate the document, create hierarchy and "links" to templates -->
				<xsl:apply-templates select="//plm:InstanceGraph" mode="createHierarchy"/>
								
				<!-- Create main scheduler -->
				<xsl:element name="scheduler"></xsl:element>
			</xsl:element>
			
			<!-- Create templates (if needed) -->
			<xsl:if test="fun:containsTemplates() = true()">
				<xsl:element name="templates">
					<!-- Iterate the document and createTemplates -->
					<xsl:apply-templates select="//plm:InstanceGraph" mode="createTemplates"/>
				</xsl:element>			
			</xsl:if>
			
		</xsl:element>
	</xsl:template>

	<xsl:template match="plm:InstanceGraph" mode="createHierarchy">
		<!-- Finding root DEBB -->
		<xsl:variable name="rootRef" select="@rootRefs" />
		<xsl:apply-templates select="plm:ProductInstance[@id=$rootRef]" mode="createHierarchy"/>
	</xsl:template>

	<xsl:template match="plm:InstanceGraph" mode="createTemplates">
		<xsl:apply-templates select="//plm:ProductRevisionView[plm:UserData/plm:UserValue/@title='DEBBLevel' and plm:UserData/plm:UserValue/@value=$templateStartLevel]" mode="createTemplates"/>
	</xsl:template>

	<xsl:template match="plm:ProductInstance" mode="createHierarchy">
		<xsl:variable name="partRef" select="@partRef" />
		<!-- Get rid of # at the beginning -->
		<xsl:variable name="productInstanceId" select="translate($partRef, '#', '')" />

		<xsl:if test="not (starts-with(@name, 'Sensor') or starts-with(@name, 'Heatsink') or starts-with(@name, 'Outlet'))">
			<!-- We have to get rid of sensors, heatsinks, outlets, which make only problems here -->
			<xsl:element name="computingResource">
				<xsl:attribute name="name"><xsl:value-of select="@name" /></xsl:attribute>


				<!-- Find corresponding ProductRevisionView and let it translate itself -->
				<xsl:apply-templates select="//plm:ProductRevisionView[@id=$productInstanceId]" mode="createHierarchy"/>

				<!-- Add hostnames, labels, locations, sequences, etc. -->
				<xsl:variable name="debbHostname"
					select="plm:UserData/plm:UserValue[@title='hostname']/@value" />
				<xsl:if test="$debbHostname != ''">
					<xsl:element name="parameter">
						<xsl:attribute name="name">hostname</xsl:attribute>
						<xsl:element name="value">
							<xsl:value-of select="$debbHostname" />
						</xsl:element>
					</xsl:element>
				</xsl:if>
				<xsl:variable name="debbLabel"
					select="plm:UserData/plm:UserValue[@title='label']/@value" />
				<xsl:if test="$debbLabel != ''">
					<xsl:element name="parameter">
						<xsl:attribute name="name">label</xsl:attribute>
						<xsl:element name="value">
							<xsl:value-of select="$debbLabel" />
						</xsl:element>
					</xsl:element>
				</xsl:if>
				<xsl:variable name="debbLocation"
					select="plm:UserData/plm:UserValue[@title='location']/@value" />
				<xsl:if test="debbLocation != ''">
					<xsl:element name="parameter">
						<xsl:attribute name="name">location</xsl:attribute>
						<xsl:element name="value">
							<xsl:value-of select="$debbLocation" />
						</xsl:element>
					</xsl:element>
				</xsl:if>
				<xsl:variable name="debbSequence"
					select="plm:UserData/plm:UserValue[@title='sequence']/@value" />
				<xsl:if test="$debbSequence != ''">
					<xsl:element name="parameter">
						<xsl:attribute name="name">sequence</xsl:attribute>
						<xsl:element name="value">
							<xsl:value-of select="$debbSequence" />
						</xsl:element>
					</xsl:element>
				</xsl:if>	
				
							
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	
	<xsl:template match="plm:ProductRevisionView" mode="createHierarchy">

		<!-- instanceRefs contain space separated ids of nested DEBBs -->
		<xsl:variable name="tokenizedText" select="tokenize(@instanceRefs, ' ')" />
	
		<!-- DEBBLevel -->
		<xsl:variable name="debbLevel"
			select="plm:UserData/plm:UserValue[@title='DEBBLevel']/@value" />
		<!-- DEBBComponentId -->
		<xsl:variable name="debbComponentId"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentId']/@value" />

		<xsl:variable name="debbComponentFile"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentFile']/@value" />
	
		<xsl:attribute name="class">
			<xsl:value-of select="$debbLevel"/>
		</xsl:attribute>
		
		<xsl:choose>
			<xsl:when test="$debbLevel = $templateStartLevel">
				<!-- Change into template. The rest of the hierarchy will be in a corresponding template. -->
				<xsl:element name="templateId">
					<xsl:value-of select="$debbLevel"/>
					<xsl:text>-</xsl:text>
					<xsl:value-of select="$debbComponentId"/>
				</xsl:element>		
			</xsl:when>
			<xsl:otherwise>
			
				<!-- Read information from DEBBComponent file -->	
				<xsl:choose>
					 <!-- DEBB Components file is defined  -->
					<xsl:when test="$debbComponentFile != ''">
						<xsl:comment>
							DEBB Component file is <xsl:value-of select="$debbComponentFile" />
						</xsl:comment>
						<xsl:variable name="nestedDoc" select="document($debbComponentFile)" />
					
<!--						<xsl:value-of select="fun:translateDEBBComponent($nestedDoc, $debbComponentFile)"/>-->

						<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]">
							<xsl:with-param name="computingResourceClasses">
								<xsl:value-of select="$computingResourceClasses" />
							</xsl:with-param>
							<xsl:with-param name="resourceUnitClasses">
								<xsl:value-of select="$resourceUnitClasses" />
							</xsl:with-param>
						</xsl:apply-templates>
					
					</xsl:when>
					 <!-- DEBB Components file is not defined. Use default one.  -->
					<xsl:otherwise>
						<xsl:comment>
							DEBB Component file is <xsl:value-of select="$defaultDEBBComponentsFileName" />
						</xsl:comment>
						<xsl:variable name="nestedDoc"
							select="document($defaultDEBBComponentsFileName)" />
		
<!--							<xsl:value-of select="fun:translateDEBBComponent($nestedDoc, $debbComponentFile)"/>-->
							
						<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]">
							<xsl:with-param name="computingResourceClasses">
								<xsl:value-of select="$computingResourceClasses" />
							</xsl:with-param>
							<xsl:with-param name="resourceUnitClasses">
								<xsl:value-of select="$resourceUnitClasses" />
							</xsl:with-param>
						</xsl:apply-templates>
		
					</xsl:otherwise>
				</xsl:choose>			
			
				<!-- Iterate children and ask them to translate themselves -->
				<xsl:for-each select="1 to count($tokenizedText)">
					<xsl:variable name="index" select="." />
					<xsl:apply-templates
						select="$rootDoc//plm:ProductInstance[@id=$tokenizedText[$index]]" mode="createHierarchy"/>
				</xsl:for-each>			
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="plm:ProductRevisionView" mode="createTemplates">
	
		<!-- instanceRefs contain space separated ids of nested DEBBs -->
		<xsl:variable name="tokenizedText" select="tokenize(@instanceRefs, ' ')" />
	
		<xsl:variable name="debbLevel"
			select="plm:UserData/plm:UserValue[@title='DEBBLevel']/@value" />
			
		<xsl:variable name="debbComponentId"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentId']/@value" />

		<xsl:variable name="debbComponentFile"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentFile']/@value" />
	
		<xsl:if test="$debbLevel">
			<xsl:element name="computingResourceTemplate">
				<xsl:attribute name="name">
					<xsl:value-of select="$debbLevel"/>
					<xsl:text>-</xsl:text>
					<xsl:value-of select="$debbComponentId"/>
				</xsl:attribute>
				<xsl:attribute name="class">
					<xsl:value-of select="$debbLevel"/>
				</xsl:attribute>
				
				<!-- Read information from DEBBComponent file -->	
				<xsl:choose>
					 <!-- DEBB Components file is defined  -->
					<xsl:when test="$debbComponentFile != ''">
						<xsl:comment>
							DEBB Component file is <xsl:value-of select="$debbComponentFile" />
						</xsl:comment>
						<xsl:variable name="nestedDoc" select="document($debbComponentFile)" />
					
<!--						<xsl:value-of select="fun:translateDEBBComponent($nestedDoc, $debbComponentFile)"/>-->

						<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]">
							<xsl:with-param name="computingResourceClasses">
								<xsl:value-of select="$computingResourceClasses" />
							</xsl:with-param>
							<xsl:with-param name="resourceUnitClasses">
								<xsl:value-of select="$resourceUnitClasses" />
							</xsl:with-param>
						</xsl:apply-templates>
					
					</xsl:when>
					 <!-- DEBB Components file is not defined. Use default one.  -->
					<xsl:otherwise>
						<xsl:comment>
							DEBB Component file is <xsl:value-of select="$defaultDEBBComponentsFileName" />
						</xsl:comment>
						<xsl:variable name="nestedDoc"
							select="document($defaultDEBBComponentsFileName)" />
		
<!--							<xsl:value-of select="fun:translateDEBBComponent($nestedDoc, $debbComponentFile)"/>-->
							
						<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]">
							<xsl:with-param name="computingResourceClasses">
								<xsl:value-of select="$computingResourceClasses" />
							</xsl:with-param>
							<xsl:with-param name="resourceUnitClasses">
								<xsl:value-of select="$resourceUnitClasses" />
							</xsl:with-param>
						</xsl:apply-templates>
		
					</xsl:otherwise>
				</xsl:choose>
				
				<!-- Iterate children and ask them to translate themselves -->
				<xsl:for-each select="1 to count($tokenizedText)">
					<xsl:variable name="index" select="." />
					<xsl:apply-templates
						select="$rootDoc//plm:ProductInstance[@id=$tokenizedText[$index]]" mode="createHierarchy"/>
				</xsl:for-each>			
							
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<xsl:function name="fun:translateDEBBComponent">
		<xsl:param name="nestedDoc"/>
		<xsl:param name="debbComponentId"/>
	
		<xsl:text>BEGIN</xsl:text>
		debbComponentId=<xsl:value-of select="$debbComponentId"/>
		<xsl:comment>
			<xsl:text>translateDEBBComponent(</xsl:text>
			<xsl:value-of select="$nestedDoc/ComponentId"/>
			<xsl:text>, </xsl:text>
			<xsl:value-of select="$debbComponentId"/>
			<xsl:text>)</xsl:text>
		</xsl:comment>
		
		<!-- Does not work :((( -->
		<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]">
			<xsl:with-param name="computingResourceClasses">
				<xsl:value-of select="$computingResourceClasses" />
			</xsl:with-param>
			<xsl:with-param name="resourceUnitClasses">
				<xsl:value-of select="$resourceUnitClasses" />
			</xsl:with-param>
		</xsl:apply-templates>

		<xsl:text>END</xsl:text>	
	</xsl:function>		
	
	<xsl:function name="fun:containsTemplates">
		<xsl:variable name="productRevisionViews" select="$rootDoc//plm:ProductRevisionView[plm:UserData/plm:UserValue/@title='DEBBLevel' and plm:UserData/plm:UserValue/@value=$templateStartLevel]"/>
		<xsl:variable name="numberOfTemplates" select="count($productRevisionViews)"/>

		<xsl:value-of select="$numberOfTemplates &gt; 0"/>
	</xsl:function>	
	
</xsl:stylesheet>

